"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyzerOutput = void 0;
const vscode = require("vscode");
const MessageSender_1 = require("../utilities/MessageSender");
class AnalyzerOutput {
    constructor() {
        this._warningsBuffer = [];
        this._newLineRegex = /\r\n?|\n\r?/gm;
    }
    get warningsBuffer() {
        return this._warningsBuffer;
    }
    setAnalysisOutput(output, onProgressChange = undefined) {
        const stdoutStrings = output.toString().trim().split(this._newLineRegex);
        stdoutStrings.forEach((stdoutString) => {
            const warning = this.tryGetWarningFromOutputLine(stdoutString);
            if (warning) {
                this._warningsBuffer.push(warning);
                return;
            }
            const noLicenseMessage = this.tryGetNoLicenseMessage(stdoutString);
            if (noLicenseMessage) {
                const licenseWarning = {
                    falseAlarm: false,
                    level: 0,
                    code: "",
                    message: noLicenseMessage,
                    positions: [],
                };
                this._showLicenseMessage(noLicenseMessage);
                this._warningsBuffer.push(licenseWarning);
                return;
            }
            let progress = this.tryGetProgressFromOutputLine(stdoutString);
            if (progress) {
                progress = Math.floor(progress);
                onProgressChange === null || onProgressChange === void 0 ? void 0 : onProgressChange(progress);
                return;
            }
        });
    }
    clearWarningsBuffer() {
        this._warningsBuffer = [];
    }
    _showLicenseMessage(stdoutString) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield MessageSender_1.MessageSender.send(`${stdoutString + !stdoutString.endsWith(".") ? "." : ""} You can enter license in settings.`, "error", false, "Go to settings");
            if (response === "Go to settings") {
                vscode.commands.executeCommand("pvsStudio.showSettings", {
                    elemToShow: "license-name-input",
                });
            }
        });
    }
}
exports.AnalyzerOutput = AnalyzerOutput;
//# sourceMappingURL=AnalyzerOutput.js.map