"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CMakeAnalyzerOutput = void 0;
const AnalyzerOutput_1 = require("../AnalyzerOutput");
class CMakeAnalyzerOutput extends AnalyzerOutput_1.AnalyzerOutput {
    constructor() {
        super(...arguments);
        this._suppressedCount = 0;
        this._suppressedCountMessageRegex = /Suppressed messages: (?<count>[0-9]+)/gm;
    }
    get suppressedCount() {
        return this._suppressedCount;
    }
    setSuppressionOutput(output) {
        const stdoutStrings = output.toString().trim().split(this._newLineRegex);
        stdoutStrings.forEach((outputLine) => {
            const count = this.tryGetSupressedCountFromOutputLine(outputLine);
            if (count !== undefined) {
                this._suppressedCount = count;
            }
        });
    }
    tryGetSupressedCountFromOutputLine(outputLine) {
        const match = this._suppressedCountMessageRegex.exec(outputLine);
        if (match) {
            return parseInt(match.groups.count);
        }
    }
    tryGetWarningFromOutputLine(outputLine) {
        if (outputLine.startsWith("{")) {
            try {
                return JSON.parse(outputLine);
            }
            catch (_a) { }
        }
    }
    tryGetProgressFromOutputLine(outputLine) {
        if (outputLine.startsWith("[")) {
            return parseInt(outputLine.slice(1, 4).trim());
        }
    }
    tryGetNoLicenseMessage(outputLine) {
        const index = outputLine.indexOf("License was not entered");
        if (index >= 0) {
            return outputLine.substring(index);
        }
    }
}
exports.CMakeAnalyzerOutput = CMakeAnalyzerOutput;
//# sourceMappingURL=CMakeAnalyzerOutput.js.map