"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompileCommands = void 0;
const vscode = require("vscode");
const path = require("path");
const fs_1 = require("fs");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
class CompileCommands {
    constructor(buildDir) {
        this._filename = "compile_commands.json";
        this._cmakelistsFilename = "CMakeLists.txt";
        this._buildDir = buildDir;
        this._path = this._getPath();
        this._cmakelistsPath = this._getCmakelistsPath();
    }
    get path() {
        if (this._path && this._cmakelistsPath && this._isYoungerThanCmakelists()) {
            return this._path;
        }
    }
    set path(path) {
        if (path && (0, fs_1.existsSync)(path)) {
            this._path = path;
        }
    }
    updatePath() {
        this.path = this._getPath();
    }
    _getPath() {
        const _path = path.join(this._buildDir, this._filename);
        const _pathExists = (0, fs_1.existsSync)(_path);
        return _pathExists ? _path : undefined;
    }
    _getCmakelistsPath() {
        let workspaceFolder = AnalyzerUtils_1.AnalyzerUtils.getRootFolder();
        const cmakeListsPath = vscode.Uri.joinPath(workspaceFolder.uri, this._cmakelistsFilename).fsPath;
        return (0, fs_1.existsSync)(cmakeListsPath) ? cmakeListsPath : undefined;
    }
    _isYoungerThanCmakelists() {
        const compileCommandsUpdateTime = (0, fs_1.statSync)(this._path).mtime;
        const cmakelistsUpdateTime = (0, fs_1.statSync)(this._cmakelistsPath).mtime;
        return compileCommandsUpdateTime.valueOf() - cmakelistsUpdateTime.valueOf() > 0;
    }
}
exports.CompileCommands = CompileCommands;
//# sourceMappingURL=CompileCommands.js.map