"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
const fs = require("fs");
const path = require("path");
const DiagnsoticsManager_1 = require("../DiagnsoticsManager");
const PluginSettingsManager_1 = require("../../PluginSettingsManager");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const GROUPS_AND_VALUES = new Map([
    [DiagnsoticsManager_1.DiagnosticsGroup.fails, 0],
    [DiagnsoticsManager_1.DiagnosticsGroup.for64bit, 1],
    [DiagnsoticsManager_1.DiagnosticsGroup.general, 4],
    [DiagnsoticsManager_1.DiagnosticsGroup.optimization, 8],
    [DiagnsoticsManager_1.DiagnosticsGroup.customer, 16],
    [DiagnsoticsManager_1.DiagnosticsGroup.misra, 32],
    [DiagnsoticsManager_1.DiagnosticsGroup.autosar, 64],
    [DiagnsoticsManager_1.DiagnosticsGroup.owasp, 128]
]);
/** Класс для работы с .cfg плагина */
class Config {
    constructor(buildDir) {
        this._filename = "config.cfg";
        this._buildDir = buildDir;
        this._path = this._getPath();
        this._pvsSettingsToConfig = this._getPvsSettingsToConfig();
    }
    getConfigPath() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this._createConfigFile();
                return this._path;
            }
            catch (error) {
                throw error;
            }
        });
    }
    _getPath() {
        return path.join(this._buildDir, ".PVS-Studio", "prepare", this._filename);
    }
    _getPvsSettingsToConfig() {
        const pvsSettings = PluginSettingsManager_1.PluginSettingsManager.settings;
        // God bless js
        const excludedPathsString = pvsSettings.excludedPaths
            .reduce((acc, cur) => {
            return acc + `exclude-path = ${cur}\n`;
        }, ``)
            .trim();
        const groups = this._getDiagnosticsGroups();
        const analysisMode = groups.reduce((acc, cur) => {
            var _a;
            return acc + ((_a = GROUPS_AND_VALUES.get(cur)) !== null && _a !== void 0 ? _a : 0);
        }, 0);
        return {
            "timeout": `timeout=${pvsSettings.analysis.timeout.value}`,
            "exclude-path": excludedPathsString,
            "analysis-mode": `analysis-mode=${analysisMode}`,
        };
    }
    _getPvsConfigPathsToConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const configPaths = yield AnalyzerUtils_1.AnalyzerUtils.getPVSConfigFilePaths(true);
            return configPaths.map((path) => `rules-config = ${path}`).join("\n");
        });
    }
    _createConfigFile() {
        return __awaiter(this, void 0, void 0, function* () {
            const content = `
new-output-format=yes
force-stdout-output=yes
${this._pvsSettingsToConfig["exclude-path"]}
${this._pvsSettingsToConfig["analysis-mode"]}
${this._pvsSettingsToConfig["timeout"]}
${yield this._getPvsConfigPathsToConfig()}
`.trim();
            try {
                yield fs.promises.mkdir(path.dirname(this._path), { recursive: true });
                return yield fs.promises.writeFile(this._path, content, { encoding: "utf-8" });
            }
            catch (error) {
                throw error;
            }
        });
    }
    _getDiagnosticsGroups() {
        const enabledDiagnostics = DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics({ onlyEnabled: true, languages: [DiagnsoticsManager_1.DiagnosticTargetLanguage.cpp] });
        const set = new Set(enabledDiagnostics.map((rule => rule.group)));
        return Array.from(set);
    }
}
exports.Config = Config;
//# sourceMappingURL=Config.js.map