"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiagnosticsManager = exports.DiagnosticsGroup = exports.DiagnosticTargetLanguage = void 0;
const vscode = require("vscode");
const fs = require("fs");
const xml2js_1 = require("xml2js");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
var DiagnosticTargetLanguage;
(function (DiagnosticTargetLanguage) {
    DiagnosticTargetLanguage["cpp"] = "cpp";
    DiagnosticTargetLanguage["cs"] = "cs";
    DiagnosticTargetLanguage["java"] = "java";
})(DiagnosticTargetLanguage = exports.DiagnosticTargetLanguage || (exports.DiagnosticTargetLanguage = {}));
var DiagnosticsGroup;
(function (DiagnosticsGroup) {
    DiagnosticsGroup["fails"] = "fails";
    DiagnosticsGroup["general"] = "ga";
    DiagnosticsGroup["optimization"] = "op";
    DiagnosticsGroup["for64bit"] = "64";
    DiagnosticsGroup["customer"] = "cs";
    DiagnosticsGroup["misra"] = "misra";
    DiagnosticsGroup["autosar"] = "autosar";
    DiagnosticsGroup["owasp"] = "owasp";
})(DiagnosticsGroup = exports.DiagnosticsGroup || (exports.DiagnosticsGroup = {}));
class DiagnosticsManager {
    static loadDiagnostics() {
        return __awaiter(this, void 0, void 0, function* () {
            const extensionUri = vscode.extensions.getExtension("EvgeniyRyzhkov.pvs-studio-vscode").extensionUri;
            const diagnsoticsXmlUri = vscode.Uri.joinPath(extensionUri, "webview-settings", "build", "data", "diagnostics.xml");
            const diagnosticsXml = fs.readFileSync(diagnsoticsXmlUri.fsPath);
            const diagnostics = yield (0, xml2js_1.parseStringPromise)(diagnosticsXml);
            this._diagnostics = new Map();
            for (var ruleSet of diagnostics["PVS-Studio_Rules"]["RuleSet"]) {
                const languageKey = Object.keys(DiagnosticTargetLanguage).find(k => DiagnosticTargetLanguage[k] === ruleSet["$"].lang.toLowerCase());
                if (!languageKey) {
                    continue;
                }
                const language = DiagnosticTargetLanguage[languageKey];
                const rules = new Map();
                for (var rule of ruleSet["Rule"]) {
                    const groupKey = Object.keys(DiagnosticsGroup).find(k => DiagnosticsGroup[k] === rule["$"].group);
                    if (groupKey) {
                        rules.set(rule["Code"][0].toUpperCase(), {
                            code: rule["Code"][0].toUpperCase(),
                            lang: language,
                            group: DiagnosticsGroup[groupKey]
                        });
                    }
                }
                this._diagnostics.set(language, rules);
            }
        });
    }
    static isDiagnostic(item) {
        return typeof item === "object" && !Array.isArray(item) && "code" in item && "lang" in item && "group" in item;
    }
    static getDiagnostics(filters) {
        let diagnsoticByCodeMaps = [];
        if (!(filters === null || filters === void 0 ? void 0 : filters.languages)) {
            diagnsoticByCodeMaps.push(...this._diagnostics.values());
        }
        else {
            diagnsoticByCodeMaps.push(...filters.languages.map(lang => { var _a; return (_a = this._diagnostics.get(lang)) !== null && _a !== void 0 ? _a : new Map(); }));
        }
        let diagnostics = [];
        if (!(filters === null || filters === void 0 ? void 0 : filters.codes)) {
            for (var map of diagnsoticByCodeMaps) {
                diagnostics.push(...map.values());
            }
        }
        else {
            for (var map of diagnsoticByCodeMaps) {
                for (var code of filters.codes) {
                    code = code.toUpperCase();
                    if (map.has(code) && !diagnostics.includes(map.get(code))) {
                        diagnostics.push(map.get(code));
                    }
                }
            }
        }
        if (filters === null || filters === void 0 ? void 0 : filters.onlyEnabled) {
            const disabledDiagnostics = PluginSettingsManager_1.PluginSettingsManager.settings.disabledDiagnostics;
            diagnostics = diagnostics.filter(d1 => !disabledDiagnostics.some(d2 => d1.code.toUpperCase() === d2.code));
        }
        return diagnostics;
    }
}
exports.DiagnosticsManager = DiagnosticsManager;
//# sourceMappingURL=DiagnsoticsManager.js.map