"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getParamDescriptionComment = exports.JavaAnalyzerConfig = void 0;
const _charCountThresholdInDescriptionLine = 130;
const _documentationLink = "https://pvs-studio.com/en/docs/manual/6703/";
const _paramDescriptionKey = Symbol("paramDescription");
const _newLinePattern = /\n\r?|\r\n?/g;
class JavaAnalyzerConfig {
}
__decorate([
    paramDescription(`
The list of files and/or directories to be analyzed (absolute or relative paths that are expanded relative to the current working directory).
If this parameter is used, only the specified files will be analyzed when analyzing the entire project.
`.replace(_newLinePattern, "").trim())
], JavaAnalyzerConfig.prototype, "analyze-only", void 0);
__decorate([
    paramDescription(`The path to the suppress file which contains suppressed warnings of the analyzer. Warnings from the suppress file will not be included in the report in any subsequent project checks.`)
], JavaAnalyzerConfig.prototype, "suppress-base", void 0);
__decorate([
    paramDescription(`The flag used to enable the incremental analysis mode. In this mode, the analyzer checks only modified files, which speeds up the analysis process. The default value is false.`)
], JavaAnalyzerConfig.prototype, "incremental", void 0);
__decorate([
    paramDescription(`
The flag used to disable caching of the program metamodel. When the cache is disabled, the project model is not cached and is rebuilt each time. This flag can be useful when identifying the causes of the analyzer errors.
Disabling project metamodel caching also disables the incremental analysis (the "incremental" parameter). The default value is false.
`.replace(_newLinePattern, "").trim())
], JavaAnalyzerConfig.prototype, "disable-cache", void 0);
__decorate([
    paramDescription(`
The flag that enables the V6078 diagnostic rule that detects potential API compatibility issues between the selected Java SE versions.
The V6078 diagnostic enables you to ensure that the JDK API you are using will not be modified or will not disappear in future versions of the JDK. The default value is false.
`.replace(_newLinePattern, "").trim())
], JavaAnalyzerConfig.prototype, "compatibility", void 0);
__decorate([
    paramDescription(`The Java SE version that your application is developed on. This setting is used by the V6078 diagnostic rule if the 'compatibility' setting is enabled. The minimum value is 8. The maximum value is 14.`)
], JavaAnalyzerConfig.prototype, "source-java", void 0);
__decorate([
    paramDescription(`
The Java SE version to be checked for compatibility with the API used in your application (source-java).
The V6078 diagnostic uses this setting if the 'compatibility' setting is enabled. The minimum value is 8. The maximum value is 14
`.replace(_newLinePattern, "").trim())
], JavaAnalyzerConfig.prototype, "target-java", void 0);
__decorate([
    paramDescription(`Packages to be excluded from the compatibility analysis (the V6078 diagnostic). The V6078 diagnostic uses this setting if the 'compatibility' setting is enabled.`)
], JavaAnalyzerConfig.prototype, "exclude-packages", void 0);
__decorate([
    paramDescription(`
The parameter used to force the rebuild of the entire cached metamodel of a program.
The metamodel contains information about the program structure and data types.
Rebuilding the project metamodel can be necessary when the analyzer version is updated or if the project metamodel is corrupted.
When this parameter is true, the incremental analysis mode is disabled (the "incremental" parameter)
`.replace(_newLinePattern, "").trim())
], JavaAnalyzerConfig.prototype, "force-rebuild", void 0);
__decorate([
    paramDescription(`Enables logging of the analyzer. Possible values: ALL, TRACE, DEBUG, INFO, WARN, ERROR, OFF.`)
], JavaAnalyzerConfig.prototype, "logging", void 0);
exports.JavaAnalyzerConfig = JavaAnalyzerConfig;
function paramDescription(description) {
    return (target, property) => {
        let paramDesc = target[_paramDescriptionKey];
        if (!paramDesc) {
            paramDesc = target[_paramDescriptionKey] = new Map();
        }
        paramDesc.set(property, description);
    };
}
function getParamDescriptionComment() {
    let comment = "If necessary, set additional analysis parameters in JSON format here, for example:\n\n";
    const exampleObj = {
        "java-path": "path",
        "compatibility": true,
        "source-java": 8,
        "target-java": 11,
        "exclude-packages": ["package-name-1", "package-name-2", "package-name-3"]
    };
    const exampleJson = JSON.stringify(exampleObj, undefined, 2);
    comment += exampleJson;
    comment += "\n\nParameters description:\n\n";
    comment += "(these parameters are similar to the arguments passed to the analyzer from the command line." +
        ` More information can be found in the documentation: ${_documentationLink})\n\n`;
    comment += getParamsDescription(JavaAnalyzerConfig.prototype);
    comment = processDescription(comment, true);
    return comment;
}
exports.getParamDescriptionComment = getParamDescriptionComment;
function getParamsDescription(prototype) {
    let description = "";
    for (var parameter of prototype[_paramDescriptionKey].keys()) {
        let paramDescription = tryGetParamDescription(parameter, prototype);
        if (paramDescription) {
            if (description !== "") {
                description += "\n\n";
            }
            description += paramDescription;
        }
    }
    return description;
}
function tryGetParamDescription(parameter, prototype) {
    const descOfParams = prototype[_paramDescriptionKey];
    if (descOfParams.has(parameter)) {
        return `${parameter} - ${descOfParams.get(parameter)}`;
    }
}
function processDescription(description, isComment) {
    const descriptionChunks = [];
    let buffer = "";
    let isSplit = false;
    for (var c of description) {
        if (isSplit && c === " " || c === "\n") {
            descriptionChunks.push(buffer);
            buffer = "";
            isSplit = false;
            continue;
        }
        buffer += c;
        if (buffer.length % _charCountThresholdInDescriptionLine === 0) {
            isSplit = true;
        }
    }
    if (buffer.length > 0) {
        descriptionChunks.push(buffer);
    }
    let result = descriptionChunks.map(chunk => chunk).join(isComment ? "\n// " : "\n");
    if (isComment) {
        result = "// " + result;
    }
    return result;
}
//# sourceMappingURL=JavaAnalyzerConfig.js.map