"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MSBuildAnalyzer = void 0;
/* eslint-disable @typescript-eslint/semi */
const Analyzer_1 = require("../Analyzer");
const vscode = require("vscode");
const MSBuildAnalyzerOutput_1 = require("./MSBuildAnalyzerOutput");
const Regedit_1 = require("../Regedit");
const path = require("path");
const MSBuildAnalyzerParam_1 = require("./MSBuildAnalyzerParam");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const MSBuildAnalyzerSettingsUtils_1 = require("./MSBuildAnalyzerSettingsUtils");
const PluginSettingsManager_1 = require("../../PluginSettingsManager");
const MSBuildAnalyserRunConfigUtils_1 = require("./RunConfigUtils/MSBuildAnalyserRunConfigUtils");
const child_process_1 = require("child_process");
const xml2js = require("xml2js");
const fs = require("fs");
const saveFile_1 = require("../../utilities/saveFile");
const os = require("os");
const MessageSender_1 = require("../../utilities/MessageSender");
class MSBuildAnalyzer extends Analyzer_1.Analyzer {
    constructor() {
        super();
        //протестировать ".cpp", ".cc", ".c++", ".cxx
        this._analyzedFileExtensions = [".cs", ".razor"];
        this._analyzedProjectExtensions = [".csproj", ".vcxproj"];
        this.description = {
            analyzerName: "MSBuild projects analyzer",
            analyzedProjectsType: "MSBuild (C#, C, C++)",
            conditionForAnalysisPossibility: "There is a solution file (.sln) in the open directory",
            extensionsOfAnalyzedFiles: [...this._analyzedFileExtensions, ...this._analyzedProjectExtensions],
            needToParseDirectories: true
        };
        this.analyzedProjectsBuildSystem = "MsBuild";
        this.analysisFileInfo = { name: "Solution", extension: "sln" };
        this.endAnalysisKey = "6e22e096";
        this._solutionFolderIsNotOpenMessage = "Please open the solution folder or file from it.";
        this._createAnalysisArgsFileMessage = "A new file with analysis parameters was created.\n\nDo you want to edit it (after running the analysis again) or continue with the default parameters?";
        this._analyzerIsNotSupportedMessage = "Current version of the PVS-Studio Analyzer is not supported. Please update the analyzer.";
        MSBuildAnalyzerSettingsUtils_1.MSBuildAnalyzerSettingsUtils.resetTempSettings();
    }
    isAnalysisPossible() {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolder = AnalyzerUtils_1.AnalyzerUtils.getRootFolder();
            if (!rootFolder) {
                return false;
            }
            const slnInfoList = yield this.getSlnPaths(rootFolder);
            return slnInfoList.length > 0;
        });
    }
    getAnalyzerPath() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const platform = os.platform();
            const pathFromConfig = (_a = vscode.workspace.getConfiguration("pvsStudio").msbuildProjectsAnalyzer.path) === null || _a === void 0 ? void 0 : _a.trim();
            if (pathFromConfig && (!/^PVS-Studio_Cmd.exe$/.test(path.basename(pathFromConfig)) || !fs.existsSync(pathFromConfig))) {
                MessageSender_1.MessageSender.send(`The specified path to the ${this.description.analyzerName.toLowerCase()} is incorrect or points to a non-existent file.`, "error");
                return { path: pathFromConfig, isSupported: false };
            }
            else if (pathFromConfig) {
                return { path: pathFromConfig, isSupported: true };
            }
            if (platform === "win32") {
                try {
                    const installDir = yield (new Regedit_1.Regedit).getInstallDir();
                    if (installDir) {
                        if (!(yield this.checkAnalyzerVersionWithExecution(path.join(installDir, "PVS-Studio_Cmd.exe"), true))) {
                            MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                            return { path: path.join(installDir, "PVS-Studio_Cmd.exe"), isSupported: false };
                        }
                        else {
                            return { path: path.join(installDir, "PVS-Studio_Cmd.exe"), isSupported: true };
                        }
                    }
                }
                catch (e) {
                    if (e instanceof Error) {
                        const message = e.message;
                        new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                            const response = yield MessageSender_1.MessageSender.send(message, "error", false, "Edit setting");
                            if (response === "Edit setting") {
                                vscode.commands.executeCommand("workbench.action.openSettings", "pvsStudio.msbuildProjectsAnalyzer.path");
                            }
                            resolve();
                        }));
                    }
                }
            }
            else {
                const command = "pvs-studio-dotnet";
                const checkCommandProcess = (0, child_process_1.spawn)("which", [command], {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                });
                let path = "";
                yield new Promise((resolve) => {
                    checkCommandProcess.stdout.on("data", (stdout) => __awaiter(this, void 0, void 0, function* () {
                        path = stdout.toString().trim();
                        resolve();
                    }));
                    checkCommandProcess.on("close", (code, signal) => {
                        resolve();
                    });
                });
                if (!(yield this.checkAnalyzerVersionWithExecution(command))) {
                    MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                    return { path: path, isSupported: false };
                }
                return { path: path, isSupported: true };
            }
            return { path: "", isSupported: true };
        });
    }
    initializeAnalysisProcess(reportPanel, onAnalysisStarted, isAnalysisEnded, analyzedFiles) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const slnInfo = yield this.selectSolution();
            if (!slnInfo) {
                return;
            }
            let args = [];
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.target, slnInfo.path);
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.writeToStdout);
            const argsFromConfig = yield this.getArgsFromRunConfig(slnInfo.name);
            if (!argsFromConfig) {
                return;
            }
            onAnalysisStarted();
            if (analyzedFiles !== undefined) {
                const analyzedSimpleFiles = analyzedFiles.filter(p => this._analyzedFileExtensions.includes(path.extname(p)));
                const analyzedProjectFiles = analyzedFiles.filter(p => this._analyzedProjectExtensions.includes(path.extname(p)));
                if (isAnalysisEnded()) {
                    return;
                }
                const allAnalyzedFiles = [...new Set([...analyzedSimpleFiles, ...(yield this.getFilesFromProjects(slnInfo.path, analyzedProjectFiles))])];
                if (isAnalysisEnded()) {
                    return;
                }
                if (allAnalyzedFiles.length > 0) {
                    const tempFolder = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
                    const sourceFilePath = path.join(tempFolder, "AnalysisFiles.txt");
                    yield fs.promises.writeFile(sourceFilePath, allAnalyzedFiles.join("\n"));
                    argsFromConfig.sourceFile = sourceFilePath;
                    argsFromConfig.intermodular = false;
                }
            }
            if (!argsFromConfig.sourceFile && this.intermodularAnalysisIsEnabled !== undefined) {
                argsFromConfig.intermodular = this.intermodularAnalysisIsEnabled;
            }
            const targetSuppressionFileName = path.basename(slnInfo.name, ".sln");
            const targetSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath(`${targetSuppressionFileName}.suppress.json`);
            if (fs.existsSync(targetSuppressionFilePath)) {
                argsFromConfig.useSuppressFiles = [...((_a = argsFromConfig.useSuppressFiles) !== null && _a !== void 0 ? _a : []), targetSuppressionFilePath];
            }
            //TODO: протестировать
            (_b = argsFromConfig.rulesConfig) !== null && _b !== void 0 ? _b : (argsFromConfig.rulesConfig = (_c = (yield AnalyzerUtils_1.AnalyzerUtils.getPVSConfigFilePaths())[0]) !== null && _c !== void 0 ? _c : "");
            args.push(...MSBuildAnalyserRunConfigUtils_1.MSBuildAnalyserRunConfigUtils.parseAnalysisArgs(argsFromConfig));
            if (isAnalysisEnded()) {
                return;
            }
            const pvsSettings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const settingParams = yield this.getMSBuildAnalyzerSettingsParams(reportPanel, pvsSettings);
            const settingsPathAndCallback = yield MSBuildAnalyzerSettingsUtils_1.MSBuildAnalyzerSettingsUtils.createSettingsFileForAnalyze(settingParams);
            if (!settingsPathAndCallback) {
                return;
            }
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.settings, settingsPathAndCallback.settingsPath);
            if (pvsSettings.sourceTreeRoot) {
                args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.sourceTreeRoot, pvsSettings.sourceTreeRoot);
            }
            const output = new MSBuildAnalyzerOutput_1.MSBuildAnalyzerOutput();
            const analyzerPath = yield this.getAnalyzerPath();
            if (isAnalysisEnded()) {
                return;
            }
            const analysis = yield this.processAnalysis(analyzerPath.path, args, output, reportPanel.view);
            analysis.on("close", (_) => __awaiter(this, void 0, void 0, function* () {
                settingsPathAndCallback.onAnalysisEnded();
                this._lastAnalyzedSolutionPath = slnInfo.path;
            }));
            return analysis;
        });
    }
    initializeSuppressionProcess(warnings, reportPanel) {
        return __awaiter(this, void 0, void 0, function* () {
            let args = ["suppression", "--mode", "Suppress"];
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.target, this._lastAnalyzedSolutionPath);
            const pvsSettings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const settingParams = yield this.getMSBuildAnalyzerSettingsParams(reportPanel, pvsSettings);
            const settingsPathAndCallback = yield MSBuildAnalyzerSettingsUtils_1.MSBuildAnalyzerSettingsUtils.createSettingsFileForAnalyze(settingParams);
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.settings, settingsPathAndCallback.settingsPath);
            if (pvsSettings.sourceTreeRoot) {
                args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.sourceTreeRoot, pvsSettings.sourceTreeRoot);
            }
            const toSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getWarningsToSuppressionFilePath();
            const reportForSuppression = { version: 2, warnings: warnings };
            yield (0, saveFile_1.saveFile)(reportForSuppression, toSuppressionFilePath);
            args.push("--analyzerReport", toSuppressionFilePath);
            const targetSuppressionFileName = path.basename(this._lastAnalyzedSolutionPath, ".sln");
            const targetSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath(`${targetSuppressionFileName}.suppress.json`);
            args.push(MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.useSuppressFile, targetSuppressionFilePath);
            const logFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getSuppressionLogFilePath();
            args.push("-r");
            const analyzerPath = yield this.getAnalyzerPath();
            const process = yield this.processSuppression(analyzerPath.path, args, reportPanel.view, warnings, targetSuppressionFilePath, logFilePath);
            process.on("close", _ => {
                settingsPathAndCallback.onAnalysisEnded();
            });
            return process;
        });
    }
    getFilesFromProjects(solutionPath, projectPaths) {
        return __awaiter(this, void 0, void 0, function* () {
            if (projectPaths.length === 0) {
                return [];
            }
            let filePathsFromProjects = [];
            let isCanceled = false;
            yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const args = [MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.target, solutionPath];
                const configArgs = {
                    selectProjects: projectPaths
                };
                args.push(...MSBuildAnalyserRunConfigUtils_1.MSBuildAnalyserRunConfigUtils.parseAnalysisArgs(configArgs));
                args.push("--parseProjectsMode");
                const parseProcess = (0, child_process_1.spawn)((yield this.getAnalyzerPath()).path, args, {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                });
                let output = "";
                parseProcess.stdout.on("data", data => {
                    output += data.toString();
                });
                parseProcess.on("close", (_) => __awaiter(this, void 0, void 0, function* () {
                    var _a;
                    const parseResult = yield xml2js.parseStringPromise(output);
                    const projectEvaluationResults = (_a = parseResult === null || parseResult === void 0 ? void 0 : parseResult["SolutionEvaluationResult"]["ProjectResults"][0]["ProjectEvaluationResult"]) !== null && _a !== void 0 ? _a : [];
                    if (projectEvaluationResults.length < projectPaths.length) {
                        let message = "The following projects will not be analyzed: ";
                        const unparsedProjects = projectPaths.filter(p => !projectEvaluationResults.some(evResult => AnalyzerUtils_1.AnalyzerUtils.isPathsEquals(evResult["ProjectPath"][0], p)));
                        for (var i = 0; i < unparsedProjects.length; i++) {
                            message += path.basename(unparsedProjects[i]);
                            message += i === unparsedProjects.length - 1 ? "." : ", ";
                        }
                        message += " Make sure they are correct and included in the selected solution.";
                        MessageSender_1.MessageSender.send(message, "error");
                    }
                    for (var evaluationResult of projectEvaluationResults) {
                        filePathsFromProjects.push(...evaluationResult["SourceFilePaths"][0]["string"]);
                    }
                    resolve();
                }));
            }));
            return filePathsFromProjects;
        });
    }
    getMSBuildAnalyzerSettingsParams(reportPanel, pvsSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            const settingParams = {
                pathMasks: pvsSettings.excludedPaths || [],
                removeIntermediateFiles: pvsSettings.analysis.removeIntermediateFiles,
                threadCount: pvsSettings.analysis.threadCount,
                disableDetectableErrors: pvsSettings.disabledDiagnostics.map(d => d.code),
                saveAfterEachMarkAsFalseAlarm: pvsSettings.falseAlarms.saveFile,
                falseAlarmsEnabled: pvsSettings.falseAlarms.show,
                analysisTimeout: pvsSettings.analysis.timeout.id
            };
            return settingParams;
        });
    }
    getArgsFromRunConfig(slnName) {
        return __awaiter(this, void 0, void 0, function* () {
            const argsInfo = yield MSBuildAnalyserRunConfigUtils_1.MSBuildAnalyserRunConfigUtils.tryGetAnalysisArgsFromRunConfig(slnName);
            if (!argsInfo) {
                return;
            }
            if (argsInfo.isCreated) {
                const response = yield MessageSender_1.MessageSender.send(this._createAnalysisArgsFileMessage, "info", true, "Edit", "Continue");
                if (response === "Edit") {
                    const runConfigPath = yield MSBuildAnalyserRunConfigUtils_1.MSBuildAnalyserRunConfigUtils.getAnalyserRunConfigPath();
                    vscode.window.showTextDocument(vscode.Uri.file(runConfigPath));
                    return;
                }
                else if (response !== "Continue") {
                    return;
                }
            }
            return argsInfo.args;
        });
    }
    getSlnPaths(rootFolder) {
        return __awaiter(this, void 0, void 0, function* () {
            const slnPattern = new vscode.RelativePattern(rootFolder, "*.sln");
            const slnUri = yield vscode.workspace.findFiles(slnPattern);
            return slnUri.map(uri => uri.fsPath);
        });
    }
    selectSolution() {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolder = AnalyzerUtils_1.AnalyzerUtils.getRootFolder();
            const slnPaths = yield this.getSlnPaths(rootFolder);
            const slnInfo = slnPaths.map(p => ({ name: path.basename(p), path: p }));
            if (slnInfo.length === 1) {
                return slnInfo[0];
            }
            const selectedSolutionName = yield vscode.window.showQuickPick(slnInfo.map(info => info.name), { placeHolder: "Select solution" });
            return slnInfo.find(info => info.name === selectedSolutionName);
        });
    }
}
exports.MSBuildAnalyzer = MSBuildAnalyzer;
//# sourceMappingURL=MSBuildAnalyzer.js.map