"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MSBuildAnalyzerParamsDescription = void 0;
const MSBuildAnalyzerParam_1 = require("./MSBuildAnalyzerParam");
const MSBuildAnalyzerRunConfig_1 = require("./RunConfigUtils/MSBuildAnalyzerRunConfig");
class MSBuildAnalyzerParamsDescription {
    static getDescriptionComment() {
        let comment = "Parameters description";
        comment += "\n\nThese parameters are similar to the arguments passed to the analyzer from the command line." +
            ` More information can be found in the documentation: ${this._documentationLink}`;
        for (var parameterInfo of (0, MSBuildAnalyzerRunConfig_1.getAnalyzerArgsInfo)()) {
            let paramDescription = MSBuildAnalyzerParamsDescription.getDescription(parameterInfo.parameter, parameterInfo.propertyName);
            comment += "\n\n" + paramDescription;
        }
        comment = this.processDescription(comment, true);
        return comment;
    }
    static getDescription(parameter, propertyName) {
        var _a;
        let description = (_a = this._parametersDescriptions.find(pair => pair.parameter === parameter)) === null || _a === void 0 ? void 0 : _a.description;
        if (!description) {
            throw new Error(`Description for parameter ${parameter} is missing.`);
        }
        description = `${propertyName} - ${description}`;
        return description;
    }
    static processDescription(description, isComment) {
        const descriptionChunks = [];
        let buffer = "";
        let isSplit = false;
        for (var c of description) {
            if (isSplit && c === " " || c === "\n") {
                descriptionChunks.push(buffer);
                buffer = "";
                isSplit = false;
                continue;
            }
            buffer += c;
            if (buffer.length % this._charCountThresholdInDescriptionLine === 0) {
                isSplit = true;
            }
        }
        if (buffer.length > 0) {
            descriptionChunks.push(buffer);
        }
        let result = descriptionChunks.map(chunk => chunk.trim()).join(isComment ? "\n// " : "\n");
        if (isComment) {
            result = "// " + result;
        }
        return result;
    }
}
exports.MSBuildAnalyzerParamsDescription = MSBuildAnalyzerParamsDescription;
MSBuildAnalyzerParamsDescription._charCountThresholdInDescriptionLine = 150;
MSBuildAnalyzerParamsDescription._documentationLink = "https://pvs-studio.ru/ru/docs/manual/0035/#IDE1BD96B458";
MSBuildAnalyzerParamsDescription._parametersDescriptions = [
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.sourceFiles, description: "path to a plaintext file with a list of source files that will be analyzed." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.platform, description: "project's build platform to analyze." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.configuration, description: "project's build configuration to analyze." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.msBuildProperties, description: "set or override the specified MSBuild project-level properties. Input example: [{\"property\": \"WarningLevel\", \"value\": \"2\"}, ....]" },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.excludeDefines, description: "list of defines which will be excluded from analyzed projects (C# projects only)." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.appendDefines, description: "list of defines which will be appended to analyzed projects (C# projects only)." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.selectProjects, description: "allows to specify for analysis individual projects from analyzer's target." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.excludeProjects, description: "allows to exclude from analysis individual projects from analyzer's target." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.rulesConfig, description: "path to the diagnostics configuration file (.pvsconfig) or to the directory containing configuration files." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.dependencyRoot, description: "an optional path to directory that will be used to store all dependency caches." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.dependencyCacheSourcesRoot, description: "an optional path to directory that will be used as a root directory to generate relative paths in dependency caches. Input example: \"SkipAnalysis\"" },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.ignoreGlobalRulesConfig, description: "disable loading global pvsconfig (located in %APPDATA%/PVS-Studio)." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.useSuppressFile, description: "path to suppress file to use." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.regenerateDependencyCache, description: "regenerates compilation dependency information for each source file." },
    { parameter: MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.intermodular, description: "C++ intermodular mode performs deeper code analysis at the cost of a longer analysis time." }
];
//# sourceMappingURL=MSBuildAnalyzerParamsDescription.js.map