"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MSBuildAnalyzerSettingsUtils = void 0;
const path = require("path");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const fs = require("fs");
const xml2js = require("xml2js");
const msBuildAnalyzerSettingsTemplate_1 = require("../../data/msBuildAnalyzerSettingsTemplate");
const os = require("os");
const AnalyzerManager_1 = require("../AnalyzerManager");
class MSBuildAnalyzerSettingsUtils {
    static createSettingsFileForAnalyze(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
            const settingsId = yield this.getFreeSettingsId();
            const tempSettingsFileName = this.tempSettingsFileName.replace("{id}", settingsId.toString());
            const tempSettingsPath = path.join(tempFolderPath, tempSettingsFileName);
            const settings = yield xml2js.parseStringPromise(msBuildAnalyzerSettingsTemplate_1.msBuildAnalyzerSettingsTemplate);
            yield this.initializeSettings(settings, params);
            const xmlBuilder = new xml2js.Builder();
            const settingsXml = xmlBuilder.buildObject(settings);
            yield fs.promises.writeFile(tempSettingsPath, settingsXml);
            const analysisEndedCallback = () => __awaiter(this, void 0, void 0, function* () { yield this.removeFile(tempSettingsPath); });
            return { settingsPath: tempSettingsPath, onAnalysisEnded: analysisEndedCallback };
        });
    }
    static resetTempSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const tempSettingPaths = yield this.getTempSettingPaths();
            for (var p of tempSettingPaths) {
                yield this.removeFile(p);
            }
        });
    }
    static initializeSettings(settings, params) {
        return __awaiter(this, void 0, void 0, function* () {
            for (var propertyName of Object.keys(params)) {
                const xmlAttributeName = this.settingsParamToXmlAttributeName(propertyName);
                const paramsKey = propertyName;
                const paramValue = params[paramsKey];
                const paramType = typeof (paramValue);
                if (paramType === "boolean" || paramType === "number" || paramType === "string") {
                    settings["ApplicationSettings"][xmlAttributeName][0] = paramValue;
                }
                else if (xmlAttributeName === "PathMasks") {
                    settings["ApplicationSettings"][xmlAttributeName][0].string = params[paramsKey];
                }
                else if (xmlAttributeName === "DisableDetectableErrors") {
                    settings["ApplicationSettings"][xmlAttributeName][0] = params[paramsKey].join(" ");
                }
            }
            if (os.platform() === "win32") {
                const licenseInfo = yield AnalyzerManager_1.AnalyzerManager.getLicenseInfo();
                if (licenseInfo) {
                    settings["ApplicationSettings"]["UserName"][0] = licenseInfo.userName;
                    settings["ApplicationSettings"]["SerialNumber"][0] = licenseInfo.licenseKey;
                }
            }
        });
    }
    static settingsParamToXmlAttributeName(paramName) {
        const firstChar = paramName[0];
        return paramName.replace(firstChar, firstChar.toUpperCase());
    }
    static removeFile(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!fs.existsSync(filePath)) {
                return true;
            }
            try {
                yield fs.promises.unlink(filePath);
                return true;
            }
            catch (_a) {
                return false;
            }
        });
    }
    static getTempSettingPaths() {
        return __awaiter(this, void 0, void 0, function* () {
            const tempFodler = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath(false);
            if (!fs.existsSync(tempFodler)) {
                return [];
            }
            return fs.readdirSync(tempFodler).filter(p => this._tempSettingsFileNamePattern.test(path.basename(p)));
        });
    }
    static getFreeSettingsId() {
        return __awaiter(this, void 0, void 0, function* () {
            const tempSettingPaths = yield this.getTempSettingPaths();
            const busyids = tempSettingPaths.map(p => parseInt(this._tempSettingsFileNamePattern.exec(path.basename(p)).groups.id));
            let freeId = 1;
            while (busyids.includes(freeId)) {
                freeId++;
            }
            return freeId;
        });
    }
}
exports.MSBuildAnalyzerSettingsUtils = MSBuildAnalyzerSettingsUtils;
MSBuildAnalyzerSettingsUtils.tempSettingsFileName = "TempSettings_{id}.xml";
MSBuildAnalyzerSettingsUtils._tempSettingsFileNamePattern = /TempSetting_(?<id>\d+).xml/;
//# sourceMappingURL=MSBuildAnalyzerSettingsUtils.js.map