"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MSBuildAnalyserRunConfigUtils = void 0;
const MSBuildAnalyzerParamsDescription_1 = require("../MSBuildAnalyzerParamsDescription");
const MSBuildAnalyzerParam_1 = require("../MSBuildAnalyzerParam");
const MSBuildAnalyzerRunConfig_1 = require("./MSBuildAnalyzerRunConfig");
const fs = require("fs");
const AnalyzerUtils_1 = require("../../AnalyzerUtils");
const stripJsonComments = require("strip-json-comments");
const path = require("path");
const MessageSender_1 = require("../../../utilities/MessageSender");
class MSBuildAnalyserRunConfigUtils {
    static tryGetAnalysisArgsFromRunConfig(slnName) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const configPath = yield this.getAnalyserRunConfigPath();
            if (!fs.existsSync(configPath)) {
                const config = new MSBuildAnalyzerRunConfig_1.MSBuildAnalyzerRunConfig();
                const args = new MSBuildAnalyzerRunConfig_1.MSBuildAnalyzerArgs();
                config.analysisArguments.push({ solutionName: slnName, arguments: args });
                this.rewriteRunConfigFile(configPath, config, false);
                return { args: args, isCreated: true };
            }
            else {
                const configText = fs.readFileSync(configPath);
                let config;
                try {
                    config = JSON.parse(stripJsonComments(configText.toString()));
                }
                catch (_b) {
                    MessageSender_1.MessageSender.send(this._invalidArgsFileFormatMessage.replace(`{runConfigPath}`, configPath), "error");
                    return;
                }
                let args = (_a = config.analysisArguments.find(p => p.solutionName === slnName)) === null || _a === void 0 ? void 0 : _a.arguments;
                if (!args) {
                    args = new MSBuildAnalyzerRunConfig_1.MSBuildAnalyzerArgs();
                    config.analysisArguments.push({ solutionName: slnName, arguments: args });
                    this.rewriteRunConfigFile(configPath, config, true);
                    return { args: args, isCreated: true };
                }
                return { args: args, isCreated: false };
            }
        });
    }
    static getAnalyserRunConfigPath() {
        return __awaiter(this, void 0, void 0, function* () {
            return path.join((yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath()), this._analyserRunConfigName);
        });
    }
    static parseAnalysisArgs(args) {
        var _a;
        const result = [];
        const argsInfo = (0, MSBuildAnalyzerRunConfig_1.getAnalyzerArgsInfo)();
        for (var propertyName of Object.keys(args)) {
            const parameter = (_a = argsInfo.find(info => info.propertyName === propertyName)) === null || _a === void 0 ? void 0 : _a.parameter;
            if (!parameter) {
                continue;
            }
            const objKey = propertyName;
            if (typeof (args[objKey]) === "string") {
                const stringArg = args[objKey];
                if (!this.isEmpty(stringArg)) {
                    result.push(parameter, stringArg);
                }
            }
            else if (Array.isArray(args[objKey])) {
                let array = args[objKey];
                if (!this.isEmpty(array)) {
                    if (parameter === MSBuildAnalyzerParam_1.MSBuildAnalyzerParam.msBuildProperties) {
                        const arg = array.map(val => `${val.property}=${val.value}`).join("|");
                        result.push(parameter, arg);
                    }
                    else if (array.every(item => typeof (item) === "string")) {
                        const arg = array.map((str) => `${str}`).join(';');
                        result.push(parameter, arg);
                    }
                }
            }
            else if (args[objKey] === "true") {
                result.push(parameter);
            }
        }
        return result;
    }
    static rewriteRunConfigFile(runConfigPath, config, isExists) {
        const configJson = JSON.stringify(config, undefined, 2);
        if (isExists) {
            fs.truncate(runConfigPath, 0, () => {
                fs.writeFileSync(runConfigPath, `${configJson}\n\n\n${MSBuildAnalyzerParamsDescription_1.MSBuildAnalyzerParamsDescription.getDescriptionComment()}`);
            });
        }
        else {
            fs.writeFileSync(runConfigPath, `${configJson}\n\n\n${MSBuildAnalyzerParamsDescription_1.MSBuildAnalyzerParamsDescription.getDescriptionComment()}`);
        }
    }
    static isEmpty(value) {
        return !value || value.length === 0;
    }
}
exports.MSBuildAnalyserRunConfigUtils = MSBuildAnalyserRunConfigUtils;
MSBuildAnalyserRunConfigUtils._analyserRunConfigName = "MSBuildAnalyzerConfig.jsonc";
MSBuildAnalyserRunConfigUtils._invalidArgsFileFormatMessage = `The analysis arguments file is incorrect. Check: {runConfigPath}`;
//# sourceMappingURL=MSBuildAnalyserRunConfigUtils.js.map