"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Regedit = void 0;
const path = require("path");
const fs = require("fs");
const vscode = require("vscode");
const regedit_1 = require("regedit");
const is64Bit_1 = require("../utilities/is64Bit");
const MessageSender_1 = require("../utilities/MessageSender");
class Regedit {
    constructor() {
        this._regeditInstallPath64 = "HKLM\\SOFTWARE\\WOW6432Node\\ProgramVerificationSystems\\PVS-Studio";
        this._regeditInstallPath32 = "HKLM\\SOFTWARE\\ProgramVerificationSystems\\PVS-Studio";
        this._regeditDeinstallPath64 = "HKLM\\SOFTWARE\\WOW6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PVS-Studio_is1";
        this._regeditDeinstallPath32 = "HKLM\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PVS-Studio_is1";
        this._regeditInstallPath = (0, is64Bit_1.default)() ? this._regeditInstallPath64 : this._regeditInstallPath32;
        this._regeditDeinstallPath = (0, is64Bit_1.default)()
            ? this._regeditDeinstallPath64
            : this._regeditDeinstallPath32;
    }
    getInstallDir() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            let installDir = null;
            let wshDisabledError = null;
            try {
                const installListResult = yield regedit_1.promisified.list([this._regeditInstallPath]);
                installDir = (_c = (_b = (_a = installListResult[this._regeditInstallPath]) === null || _a === void 0 ? void 0 : _a.values) === null || _b === void 0 ? void 0 : _b.InstallDir) === null || _c === void 0 ? void 0 : _c.value;
                if (!installDir) {
                    installDir = yield this._getDeinstallDir();
                }
            }
            catch (e) {
                if (e instanceof Error && e.message.startsWith("Command failed: cscript.exe")) {
                    wshDisabledError = e;
                }
            }
            if (!installDir) {
                installDir = this._getDefaultInstallDir();
            }
            if (!installDir && wshDisabledError) {
                new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    const message = "PVS-Studio is unable to get the path to the analyzer through the Windows registry because the Windows Script Host is disabled. Enable it or specify the path explicitly in the extension settings.";
                    const response = yield MessageSender_1.MessageSender.send(message, "error", false, "Edit settings");
                    if (response === "Edit settings") {
                        vscode.commands.executeCommand("workbench.action.openSettings", "pvsStudio");
                    }
                    resolve();
                }));
            }
            return installDir;
        });
    }
    _getDeinstallDir() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore
            const deinstallListResult = yield regedit_1.promisified.list(this._regeditDeinstallPath);
            return (_c = (_b = (_a = deinstallListResult[this._regeditDeinstallPath]) === null || _a === void 0 ? void 0 : _a.values) === null || _b === void 0 ? void 0 : _b.InstallLocation) === null || _c === void 0 ? void 0 : _c.value;
        });
    }
    _getDefaultInstallDir() {
        const programFilesDir = process.env["PROGRAMFILES(X86)"];
        if (!programFilesDir) {
            return null;
        }
        const pvsStudioDefaultDir = path.join(programFilesDir, "PVS-Studio");
        if (fs.existsSync(pvsStudioDefaultDir)) {
            return pvsStudioDefaultDir;
        }
        else {
            return null;
        }
    }
}
exports.Regedit = Regedit;
//# sourceMappingURL=Regedit.js.map