"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const vscode_1 = require("vscode");
const ReportPanel_1 = require("./panels/ReportPanel");
const SettingsPanel_1 = require("./panels/SettingsPanel");
const AnalyzerManager_1 = require("./analyzer/AnalyzerManager");
const DiagnsoticsManager_1 = require("./analyzer/DiagnsoticsManager");
const PluginSettingsManager_1 = require("./PluginSettingsManager");
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        yield DiagnsoticsManager_1.DiagnosticsManager.loadDiagnostics();
        yield PluginSettingsManager_1.PluginSettingsManager.loadOrCreateSettings();
        // Инициализируем вкладку самого плагина
        const reportTableProvider = new ReportPanel_1.ReportPanel(context.extensionUri);
        context.subscriptions.push(vscode.window.registerWebviewViewProvider(ReportPanel_1.ReportPanel.viewType, reportTableProvider, {
            webviewOptions: {
                retainContextWhenHidden: true,
            },
        }));
        // Инициализируем вкладку настроек
        context.subscriptions.push(vscode.commands.registerCommand("pvsStudio.showSettings", (params) => {
            SettingsPanel_1.SettingsPanel.createOrShow(context.extensionUri, params);
        }));
        // открыть панель PVS-Studio
        context.subscriptions.push(vscode.commands.registerCommand("pvsStudio.showWindow", () => __awaiter(this, void 0, void 0, function* () {
            yield vscode.commands.executeCommand("PVSPanel.focus");
        })));
        // Команда загрузки отчета
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.loadReport", () => __awaiter(this, void 0, void 0, function* () {
            vscode.commands.executeCommand("workbench.view.extension.PVSPanel").then(() => {
                reportTableProvider.getReportJson();
            });
        })));
        // Команда простановки Favorite true
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.markAsFavorite", () => __awaiter(this, void 0, void 0, function* () {
            reportTableProvider.toggleFavorite(true);
        })));
        // Команда простановки Favorite false
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.unmarkFavorite", () => __awaiter(this, void 0, void 0, function* () {
            reportTableProvider.toggleFavorite(false);
        })));
        // Команда простановки False Alarm true
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.markAsFalseAlarm", () => __awaiter(this, void 0, void 0, function* () {
            reportTableProvider.toggleFalseAlarm(true);
        })));
        // Команда простановки False Alarm false
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.unmarkFalseAlarm", () => __awaiter(this, void 0, void 0, function* () {
            reportTableProvider.toggleFalseAlarm(false);
        })));
        // Команда простановки False Alarm false
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.excludeDiagnostic", () => __awaiter(this, void 0, void 0, function* () {
            reportTableProvider.excludeDiagnostic();
        })));
        // Команда запуска межмодульного анализа
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.runIntermodularAnalysis", () => __awaiter(this, void 0, void 0, function* () {
            yield AnalyzerManager_1.AnalyzerManager.runAnalysis(reportTableProvider, true);
        })));
        // Команда запуска обычного анализа
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.runRegularAnalysis", () => __awaiter(this, void 0, void 0, function* () {
            yield AnalyzerManager_1.AnalyzerManager.runAnalysis(reportTableProvider, false);
        })));
        // Команда запуска анализа одного файла
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.runFileAnalysis", (targetFile) => __awaiter(this, void 0, void 0, function* () {
            yield AnalyzerManager_1.AnalyzerManager.runAnalysis(reportTableProvider, false, [targetFile.fsPath]);
        })));
        // Команда запуска анализа нескольких файлов
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.runFilesAnalysis", (targetFile, specificInfo) => __awaiter(this, void 0, void 0, function* () {
            let targetPaths = [];
            // свойство groupId используется только для того, чтобы определить, что был запрошен анализ группы вкладок
            // для идентификации конкретной группы его использовать нельзя, так это свойство сломано и зачастую содержит неверный id
            // вместо этого получаем текущую активную группу вкладок (менюшка которая содержит кнопку для анализа группы доступна только для активной группы)
            if (specificInfo === null || specificInfo === void 0 ? void 0 : specificInfo.hasOwnProperty("groupId")) {
                const targetGroup = vscode.window.tabGroups.activeTabGroup;
                targetPaths.push(...targetGroup.tabs.map(tab => { var _a, _b; return (_b = (_a = tab.input) === null || _a === void 0 ? void 0 : _a.uri) === null || _b === void 0 ? void 0 : _b.fsPath; }).filter(p => p));
            }
            else if (specificInfo && Array.isArray(specificInfo)) {
                targetPaths.push(...specificInfo.map((f) => f.fsPath).filter(p => p));
            }
            else if (!specificInfo && targetFile.fsPath) {
                targetPaths.push(targetFile.fsPath);
            }
            yield AnalyzerManager_1.AnalyzerManager.runAnalysis(reportTableProvider, false, targetPaths);
        })));
        // Команда запуска подавления
        context.subscriptions.push(vscode_1.commands.registerCommand("pvsStudio.suppressMessages", () => __awaiter(this, void 0, void 0, function* () {
            AnalyzerManager_1.AnalyzerManager.requestSuppressionWarrnings(reportTableProvider);
        })));
        yield AnalyzerManager_1.AnalyzerManager.onRunningExtension();
    });
}
exports.activate = activate;
function deactivate(context) { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map