"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportPanel = void 0;
const vscode = require("vscode");
const getJsonFromFile_1 = require("../utilities/getJsonFromFile");
const getUri_1 = require("../utilities/getUri");
const saveFile_1 = require("../utilities/saveFile");
const pathNode = require("path");
const os = require("os");
const fs = require("fs");
const hash_1 = require("../utilities/hash");
const AnalyzerManager_1 = require("../analyzer/AnalyzerManager");
const AnalyzerUtils_1 = require("../analyzer/AnalyzerUtils");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
const DiagnsoticsManager_1 = require("../analyzer/DiagnsoticsManager");
const MessageSender_1 = require("../utilities/MessageSender");
const dotenv = require('dotenv');
dotenv.config();
class ReportPanel {
    get view() {
        return this._view;
    }
    get isBlocked() {
        return this._panelIsBlocked;
    }
    constructor(_uri) {
        this._uri = _uri;
        this.reportVersion = 4;
        this.fsPath = "";
        this.filesWithChangedFalseAlarm = new Set();
        this.focusedRows = [];
        this.isSourceTreeRoot = false;
        this.fileLastOpened = {
            file: "",
            line: -1,
        };
        this._reportStateFileName = "ReportState.json";
        this._panelStateFileName = "WindowState.json";
        this._isLoadingReport = false;
        this._panelIsBlocked = false;
        this.extensionUri = _uri;
    }
    resolveWebviewView(webviewView, context, _token) {
        this._view = webviewView;
        webviewView.webview.options = {
            // Allow scripts in the webview
            enableScripts: true,
            localResourceRoots: [this.extensionUri],
        };
        webviewView.webview.html = this._getHtmlForWebview(webviewView.webview);
        // The build folder from the React build output
        const buildUri = (0, getUri_1.getUri)(webviewView.webview, this.extensionUri, ["webview-ui", "build"]);
        webviewView.onDidDispose(() => {
            if (AnalyzerManager_1.AnalyzerManager.isRunAnalysis()) {
                vscode.window.showErrorMessage("Analysis was aborted due to PVS-Studio window being moved to a different tab location. Please restart the analysis.");
            }
            AnalyzerManager_1.AnalyzerManager.endAnalysis(this);
        });
        // Получение сообщений из вьюхи
        webviewView.webview.onDidReceiveMessage((message) => __awaiter(this, void 0, void 0, function* () {
            const command = message.command;
            switch (command) {
                case "savePanelState":
                    yield this.saveState(this._panelStateFileName, message.state);
                    return;
                case "saveReportState":
                    yield this.saveState(this._reportStateFileName, message.state);
                    return;
                case "deleteReportState":
                    yield this.deleteState(this._reportStateFileName);
                    return;
                case "loadState":
                    yield this.loadState();
                    return;
                case "loadReportJson":
                    yield this.getReportJson();
                    return;
                case "saveReportJson":
                    this.saveReportJson(message.data, message.isAutosave);
                    return;
                case "writeFalseAlarm":
                    this.writeFalseAlarmCommentary(message.data);
                    return;
                case "openFile":
                    this.openFile(message.file, message.openLineWithoutHash);
                    return;
                case "setFocusedRows":
                    this.focusedRows = message.focusedRows;
                    return;
                case "toggleFavorite":
                    this.toggleFavorite(message.isCheck);
                    return;
                case "toggleFalseAlarm":
                    this.toggleFalseAlarm(message.isCheck);
                    return;
                case "excludeDiagnostic": {
                    this.excludeDiagnostic();
                    return;
                }
                case "excludePath": {
                    this.excludePath(message.path);
                    return;
                }
                case "openSettings": {
                    vscode.commands.executeCommand("pvsStudio.showSettings");
                    return;
                }
                case "getSettings": {
                    if (!this._view) {
                        return;
                    }
                    this._view.webview.postMessage({
                        type: "settings",
                        settings: PluginSettingsManager_1.PluginSettingsManager.settings,
                        buildUri: buildUri,
                        fsSeparator: pathNode.sep,
                        enabledDiagnosticGroups: [...new Set(DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics({ onlyEnabled: true }).map(d => d.group))]
                    });
                    return;
                }
                case "openSite": {
                    try {
                        vscode.commands.executeCommand("simpleBrowser.show", [message.url]);
                    }
                    catch (error) {
                        console.log("тут будет отправка на фронт");
                    }
                    return;
                }
                case "copyMessage": {
                    this.copyMessage(message.visibleColumns);
                    return;
                }
                case "showMessage": {
                    MessageSender_1.MessageSender.send(message.message, message.type);
                    return;
                }
                case "toggleColumnVisibility": {
                    yield this.toggleColumnVisibility(message.colId);
                    return;
                }
                case "analyzeProject": {
                    yield AnalyzerManager_1.AnalyzerManager.runAnalysis(this);
                    return;
                }
                case "endAnalysis": {
                    AnalyzerManager_1.AnalyzerManager.endAnalysis(this);
                    return;
                }
                case "showSuppressMessagesDialog": {
                    AnalyzerManager_1.AnalyzerManager.requestSuppressionWarrnings(this);
                    return;
                }
                case "suppressWarnings": {
                    yield AnalyzerManager_1.AnalyzerManager.suppressWarnings(message.warnings, this);
                    return;
                }
            }
        }));
        PluginSettingsManager_1.PluginSettingsManager.addChangeEventListener(settings => {
            if (!this._view) {
                return;
            }
            this._view.webview.postMessage({
                type: "settings",
                settings: settings,
                buildUri: buildUri,
                fsSeparator: pathNode.sep,
                enabledDiagnosticGroups: [...new Set(DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics({ onlyEnabled: true }).map(d => d.group))]
            });
        });
    }
    saveState(stateFileName, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath(false);
            if (pvsFolderPath) {
                const stateFolderPath = pathNode.join(pvsFolderPath, "State");
                const stateFilePath = pathNode.join(stateFolderPath, stateFileName);
                yield (0, saveFile_1.saveFile)(state, stateFilePath);
            }
        });
    }
    deleteState(stateFileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath(false);
            if (pvsFolderPath) {
                const stateFolderPath = pathNode.join(pvsFolderPath, "State");
                const stateFilePath = pathNode.join(stateFolderPath, stateFileName);
                if (fs.existsSync(stateFilePath)) {
                    yield fs.promises.unlink(stateFilePath);
                }
            }
        });
    }
    loadState() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath(false);
            if (pvsFolderPath) {
                const stateFolderPath = pathNode.join(pvsFolderPath, "State");
                this._view.webview.postMessage({ type: "isLoading", isLoading: true, processId: "loadState" });
                const reportStateFilePath = pathNode.join(stateFolderPath, this._reportStateFileName);
                const panelStateFilePath = pathNode.join(stateFolderPath, this._panelStateFileName);
                let panelState = yield (0, getJsonFromFile_1.getJsonFromFile)(panelStateFilePath).catch(e => { });
                if (!this._isLoadingReport && ((_a = panelState.loadableProcessIds) === null || _a === void 0 ? void 0 : _a.includes("loadReportJson"))) {
                    panelState.loadableProcessIds = panelState.loadableProcessIds.filter((item) => item !== "loadReportJson");
                }
                let reportState = fs.existsSync(reportStateFilePath) && !((_b = panelState.loadableProcessIds) === null || _b === void 0 ? void 0 : _b.includes("loadReportJson")) ? yield (0, getJsonFromFile_1.getJsonFromFile)(reportStateFilePath).catch(e => { }) : {};
                reportState !== null && reportState !== void 0 ? reportState : (reportState = {});
                yield this._view.webview.postMessage({ type: "setState", state: Object.assign(Object.assign({}, reportState), panelState) });
                this._view.webview.postMessage({ type: "isLoading", isLoading: false, processId: "loadState" });
            }
        });
    }
    /**
     * Дает пользователю выбрать файл отчета, который затем преобразует в JSON объект
     * @returns {void}
     */
    getReportJson() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._view) {
                return;
            }
            // let project;
            // let codeModel;
            // const api = await getCMakeToolsApi(Version.v1);
            // if (api) {
            //   const wf = vscode.workspace.workspaceFolders;
            //   if (!wf) {
            //     return;
            //   }
            //   project = (await api.getProject(wf[0].uri)) as Project;
            //   if (!project) {
            //     return;
            //   }
            //   codeModel = project.codeModel;
            //   console.log("hererererere");
            // }
            // Выбираем файл через окно выбора файлов
            const fileObj = yield vscode.window.showOpenDialog({ filters: { json: ["json"] } });
            // Если файл не выбран
            if (!fileObj) {
                return;
            }
            // Получаем путь до файла
            let { path } = fileObj[0];
            if ((path.startsWith("\\") || path.startsWith("/")) && os.platform() === "win32") {
                path = path.slice(1);
            }
            this.fsPath = path;
            // Проверяем, .json ли это
            if (path.split(".")[path.split(".").length - 1] !== "json") {
                vscode.window.showErrorMessage("PVS-Studio: Incorrect format of PVS-Studio JSON report.");
                return;
            }
            if (this._view) {
                this._view.webview.postMessage({ type: "isLoading", isLoading: true, processId: "loadReportJson" });
                this.blockingPanel(true);
                vscode.commands.executeCommand("PVSPanel.focus");
            }
            AnalyzerManager_1.AnalyzerManager.endAnalysis(this);
            this._isLoadingReport = true;
            // Проверяем содержимое .json на json-овость
            try {
                const json = yield (0, getJsonFromFile_1.getJsonFromFile)(path);
                // Попытка выкинуть ошибку, если отчет не соответствует ожидаемому
                if (!(json === null || json === void 0 ? void 0 : json.warnings)) {
                    MessageSender_1.MessageSender.send("Incorrect format of PVS-Studio report.", "error");
                    this._view.webview.postMessage({ type: "isLoading", isLoading: false, processId: "loadReportJson" });
                    return;
                }
                this.reportVersion = json.version;
                if (this.reportVersion < 3 && json.warnings.length) {
                    json.warnings = json.warnings.map((warning) => {
                        var _a;
                        for (let i = 0; i < ((_a = warning.projects) === null || _a === void 0 ? void 0 : _a.length); i++) {
                            warning.projects[i] = this.tryGetProjectNameWithoutTargetingFramework(warning.projects[i]);
                        }
                        return warning;
                    });
                }
                json.warnings = AnalyzerUtils_1.AnalyzerUtils.prepareWarnings(json.warnings);
                AnalyzerManager_1.AnalyzerManager.warningsSuppressionDisable();
                // Отправляем json вьюхе
                if (this._view) {
                    (_b = (_a = this._view).show) === null || _b === void 0 ? void 0 : _b.call(_a, true); // `show` is not implemented in 1.49 but is for 1.50 insiders
                    yield this._view.webview.postMessage({ type: "reportJson", reportJson: json });
                }
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            this._view.webview.postMessage({ type: "isLoading", isLoading: false, processId: "loadReportJson" });
            this.blockingPanel(false);
            this._isLoadingReport = false;
        });
    }
    blockingPanel(isEnable) {
        var _a;
        (_a = this._view) === null || _a === void 0 ? void 0 : _a.webview.postMessage({ type: "blocked", isBlocked: isEnable });
        vscode.commands.executeCommand("setContext", "pvs-studio.blockingGridFunctionality", isEnable);
        this._panelIsBlocked = isEnable;
    }
    tryGetProjectNameWithoutTargetingFramework(projectName) {
        let regexp = new RegExp("(.*\\(netcoreapp\\d+.\\d+\\)$)|(.*\\(netstandard\\d+.\\d+\\)$)|(.*\\(net\\d+\\)$)|(.*\\(net\\d+.\\d+\\)$)");
        if (!projectName.length || !projectName.endsWith(')') || !regexp.test(projectName)) {
            return projectName;
        }
        let indexOpeningParenthesis = projectName.lastIndexOf('(');
        if (indexOpeningParenthesis === -1) {
            return projectName;
        }
        return projectName.substring(0, indexOpeningParenthesis);
    }
    /**
     * Пишет в файлы комментарий ложного срабатывания
     * @param {IFalseAlarmData} data
     * @returns
     */
    writeFalseAlarmCommentary(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const isUseMessagesInFalseAlarms = settings.falseAlarms.message;
            const commentaryWithoutMessage = ` //-${data.code}`;
            const commentaryWithMessage = ` //-${data.code} ${data.message}`;
            const commentary = isUseMessagesInFalseAlarms
                ? commentaryWithMessage
                : commentaryWithoutMessage;
            const file = data.files[0];
            this.filesWithChangedFalseAlarm.add(file);
            const filePath = vscode.Uri.file(file.file);
            try {
                const textDocument = yield vscode.workspace.openTextDocument(file.file);
                const editor = yield vscode.window.showTextDocument(textDocument, 1, false);
                const line = yield this.getFileLineByHashSum(editor, file);
                if (!line) {
                    throw new Error("");
                }
                this.setCursorLinePosition(editor, line);
                editor.edit((edit) => {
                    // Вычитаем 1, тк отсчет строк у vscode api идет с нуля
                    // 1000 - это условно очень большая строка, мы хотим выделить весь текст не зная его ширины
                    const lineSelection = new vscode.Selection(line - 1, 0, line - 1, 1000);
                    const lineText = editor.document.getText(lineSelection);
                    if (data.isChecked) {
                        // Проставляем комментарий
                        edit.replace(lineSelection, `${lineText}${commentary}`);
                    }
                    else {
                        // Убираем комментарий
                        // Учитываем вариант, что пользователь мог сначала проставить False Alarm с мессаджем,
                        // а затем убрать мессадж, с помощью двойного replace
                        const stringWithoutCommentary = lineText
                            .replace(commentaryWithMessage, "")
                            .replace(commentaryWithoutMessage, "");
                        edit.replace(lineSelection, stringWithoutCommentary);
                    }
                });
                if (settings.falseAlarms.saveFile) {
                    textDocument.save();
                }
                return true;
            }
            catch (error) {
                // Если в отчете есть SourceTreeRoot, то предлагаем поменять в настройках
                if (this.isSourceTreeRoot) {
                    const positiveAnswer = "Go to Settings";
                    const filePathWithoutSTR = file.file.replace("|?|", "");
                    const answer = yield vscode.window.showErrorMessage(`Cannot open file with following relative path: ${filePathWithoutSTR}, try changing STR in settings.`, positiveAnswer);
                    if (answer === positiveAnswer) {
                        vscode.commands.executeCommand("pvsStudio.showSettings", {
                            elemToShow: "source-tree-root",
                        });
                    }
                }
                else {
                    vscode.window.showErrorMessage(error.message);
                }
                return false;
            }
        });
    }
    _getReportSavingPath() {
        var _a;
        const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
        if (settings.sourceTreeRoot.length !== 0) {
            return settings.sourceTreeRoot;
        }
        const platform = os.platform();
        if (platform === "win32") {
            return process.env["USERPROFILE"] + "\\Documents\\";
        }
        else {
            return (_a = process.env["HOME"]) !== null && _a !== void 0 ? _a : "~/";
        }
    }
    /**
     * Сохраняет файл json на диске в папке, которую укажет юзер
     * @param data
     */
    saveReportJson(data, isAutosave) {
        return __awaiter(this, void 0, void 0, function* () {
            let path;
            console.log();
            if (!isAutosave) {
                const saveFileInfo = yield vscode.window.showSaveDialog({
                    defaultUri: vscode.Uri.file(this._getReportSavingPath()),
                    filters: {
                        json: ["json"],
                    },
                });
                // @ts-ignore
                path = saveFileInfo === null || saveFileInfo === void 0 ? void 0 : saveFileInfo.fsPath;
            }
            else {
                path = this.fsPath
                    .split(".")
                    .map((a, index) => {
                    if (index === 0) {
                        return a + "_autosave";
                    }
                    return a;
                })
                    .join(".");
            }
            if (!path) {
                if (this._view) {
                    this._view.webview.postMessage({ type: "fileIsSaved" });
                }
                return;
            }
            const response = yield (0, saveFile_1.saveFile)(data, path);
            // Если пришла пустота - файл сохранился
            if (!response) {
                if (isAutosave) {
                    return;
                }
                vscode.window.showInformationMessage("Report has been saved.");
                // @ts-ignore
                this._view.webview.postMessage({ type: "setIsDataChanged", isDataChanged: false });
                // Когда ошибка, приходит объект с полем message
            }
            else {
                //@ts-ignore
                vscode.window.showErrorMessage(response.message);
            }
            if (this._view) {
                this._view.webview.postMessage({ type: "fileIsSaved" });
            }
        });
    }
    postMessageAutosave() {
        if (this._view) {
            this._view.webview.postMessage({ type: "autosave" });
        }
    }
    toggleFavorite(isCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.focusedRows.length === 0) {
                vscode.window.showErrorMessage("No rows selected");
                return;
            }
            if (this.focusedRows.length >= 100) {
                const positiveAnswer = "Yes";
                const negativeAnswer = "No";
                const response = yield vscode.window.showWarningMessage("Too many rows selected, are you sure you want dis? It can be catastrophy :(", positiveAnswer, negativeAnswer);
                if (response === negativeAnswer) {
                    return;
                }
            }
            if (this._view) {
                this._view.webview.postMessage({
                    type: "toggleFavorite",
                    focusedRows: this.focusedRows,
                    isCheck,
                });
            }
        });
    }
    toggleFalseAlarm(isCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._view) {
                return;
            }
            if (this.focusedRows.length === 0) {
                vscode.window.showErrorMessage("No rows selected");
                return;
            }
            if (this.focusedRows.length >= 100) {
                const positiveAnswer = "Yes";
                const negativeAnswer = "No";
                const response = yield vscode.window.showWarningMessage("Too many rows selected, are you sure you want dis? It can be catastrophy :(", positiveAnswer, negativeAnswer);
                if (response === negativeAnswer) {
                    return;
                }
            }
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const sourceTreeRoot = settings.sourceTreeRoot || "";
            const updatedRows = [];
            for (const row of this.focusedRows) {
                const falseAlarmWrittten = yield this.writeFalseAlarmCommentary({
                    files: row.positionsWithSettings.map((position) => ({
                        file: position.file.replace(/\|\?\|/g, sourceTreeRoot).replace(`${pathNode.sep}${pathNode.sep}`, pathNode.sep),
                        line: position.line,
                        navigation: position.navigation,
                    })),
                    isChecked: isCheck,
                    code: row.code,
                    message: row.message,
                });
                if (falseAlarmWrittten) {
                    updatedRows.push(row);
                }
            }
            this._view.webview.postMessage({
                type: "toggleFalseAlarm",
                focusedRows: updatedRows,
                isCheck,
            });
        });
    }
    excludeDiagnostic() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._view) {
                return;
            }
            if (this.focusedRows.length === 0) {
                vscode.window.showErrorMessage("No rows selected");
                return;
            }
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            for (var row of this.focusedRows) {
                const { code, lang } = row;
                if (AnalyzerUtils_1.AnalyzerUtils.alwaysEnabledDiagnostics.includes(code)) {
                    return;
                }
                if (!PluginSettingsManager_1.PluginSettingsManager.settings.disabledDiagnostics.some(d => d.code.toUpperCase() === code.toUpperCase())) {
                    const diagnostic = DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics({ codes: [code] })[0];
                    settings.disabledDiagnostics = [...settings.disabledDiagnostics, diagnostic];
                }
            }
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(Object.assign({}, settings));
        });
    }
    excludePath(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            let excludedPaths = (settings === null || settings === void 0 ? void 0 : settings.excludedPaths) || [];
            let isDirectory;
            try {
                const stats = yield fs.promises.stat(path);
                isDirectory = stats.isDirectory();
            }
            catch (_a) {
                pathNode.basename(path).includes(".") ? isDirectory = false : isDirectory = true;
            }
            let pathToExclude = isDirectory ? pathNode.join(path, "*") : path;
            excludedPaths = [...new Set([...excludedPaths, pathToExclude])];
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(Object.assign(Object.assign({}, settings), { excludedPaths }));
        });
    }
    copyMessage(visibleColumns) {
        if (!this._view) {
            return;
        }
        if (this.focusedRows.length === 0) {
            return;
        }
        const excludedCols = ["favorite", "falseAlarm"];
        const visibleColumnsWithoutExcluded = visibleColumns.filter((col) => !excludedCols.includes(col));
        let messages = [];
        for (const row of this.focusedRows) {
            let message = "";
            let cweAndSast = [];
            visibleColumnsWithoutExcluded.forEach((col, i) => {
                if (col === "id") {
                    return;
                }
                if (col === "code") {
                    message += row[col];
                    return;
                }
                if (col === "files") {
                    const files = row[col].split(";;");
                    let separator = files[0].indexOf('\\') === -1 ? '/' : '\\';
                    let filename = files[0].split(separator).slice(-1)[0].replaceAll(":", " ");
                    message += " " + filename;
                    return;
                }
                if (col === "cwe") {
                    if (row[col] !== 0 && row[col] !== undefined) {
                        cweAndSast.push("CWE-" + row[col]);
                    }
                    return;
                }
                if (col === "sastId") {
                    cweAndSast.push(row[col]);
                    return;
                }
                if (col === "message") {
                    cweAndSast = cweAndSast.filter(item => item === null || item === void 0 ? void 0 : item.length);
                    if (cweAndSast.length > 1) {
                        message += ' [' + cweAndSast.join(', ') + '] ' + row[col];
                    }
                    else if (cweAndSast.length > 0) {
                        message += ' [' + cweAndSast[0] + '] ' + row[col];
                    }
                    else {
                        message += ' ' + row[col];
                    }
                    return;
                }
                if (typeof row[col] === "string" || typeof row[col] === "number") {
                    message += " " + row[col];
                    return;
                }
                if (Array.isArray(row[col])) {
                    message += row[col].join(", ");
                    return;
                }
            });
            messages = [...messages, message];
        }
        this._view.webview.postMessage({
            type: "copyMessage",
            message: messages.join("\n"),
        });
    }
    toggleColumnVisibility(colId) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            if (colId === "falseAlarms") {
                settings.falseAlarms.show = !settings.falseAlarms.show;
            }
            else {
                settings.columns = settings.columns.map((col) => {
                    if (col.id === colId) {
                        return Object.assign(Object.assign({}, col), { visible: !col.visible });
                    }
                    return col;
                });
            }
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(settings);
        });
    }
    /**
     * Открывает файл в окне vscode на нужной строке
     * @param file
     */
    openFile(file, openLineWithoutHash) {
        return __awaiter(this, void 0, void 0, function* () {
            let fileToOpen;
            let indexOfFile = -1;
            if (!file.hasOwnProperty("files")) {
                fileToOpen = file;
            }
            else if (file.files.length === 1) {
                fileToOpen = file.files[0];
            }
            else {
                indexOfFile = file.files.findIndex((f) => f.file === this.fileLastOpened.file && f.line === this.fileLastOpened.line);
                if (indexOfFile >= 0) {
                    fileToOpen = file.files[indexOfFile + 1] ? file.files[indexOfFile + 1] : file.files[0];
                }
                else {
                    fileToOpen = file.files[0];
                }
            }
            this.fileLastOpened = fileToOpen;
            try {
                const textDocument = yield vscode.workspace.openTextDocument(fileToOpen.file);
                const editor = yield vscode.window.showTextDocument(textDocument, 1, false);
                let line = fileToOpen.line;
                if (fileToOpen.hasOwnProperty("navigation") && indexOfFile !== 0 && !openLineWithoutHash) {
                    // @ts-ignore
                    line = yield this.getFileLineByHashSum(editor, fileToOpen);
                    this.setCursorLinePosition(editor, line);
                }
                else {
                    this.setCursorLinePosition(editor, line);
                }
            }
            catch (error) {
                let msg = `Can't open ${fileToOpen.file}.`;
                let options = [];
                const settingsAnswer = "Go to settings";
                if (fileToOpen.file.includes("|?|")) {
                    msg += ` Try set Source tree root in the settings.`;
                    options = [...options, settingsAnswer];
                }
                const answer = yield vscode.window.showErrorMessage(msg, ...options);
                if (answer === settingsAnswer) {
                    vscode.commands.executeCommand("pvsStudio.showSettings", {
                        elemToShow: "source-tree-root",
                    });
                }
            }
        });
    }
    /**
     * Устанавливает курсор в конец указанной строки в открытом файле
     * @param editor открытый файл
     * @param line строка
     */
    setCursorLinePosition(editor, line) {
        const range = editor.document.lineAt(line - 1).range;
        // Нам не нужно выделение, но этот хак позволит переместить курсор на нужную строку
        editor.selection = new vscode.Selection(range.end, range.end);
        editor.revealRange(range, 1);
        // Фокусит наш плагин
        // vscode.commands.executeCommand("workbench.view.extension.PVSPanel");
    }
    /**
     * Выдает строку, на которой надо открыть файл, с учетом хэш суммы строки
     * @param editor Открытый файл
     * @param file Объект, где есть хэш строк
     * @returns
     */
    getFileLineByHashSum(editor, file) {
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore
            const lineHashReportCurrent = file.navigation.currentLine;
            // @ts-ignore
            const lineHashReportPrev = file.navigation.previousLine;
            // @ts-ignore
            const lineHashReportNext = file.navigation.nextLine;
            const allText = editor.document.getText();
            const allLinesCount = allText.split(/\r?\n/gm).length;
            const linesFrom = file.line - 10 > 0 ? file.line - 10 : 0 + 1;
            const linesTo = file.line + 10 < allLinesCount ? file.line + 10 : allLinesCount - 2;
            const lineCur = file.line - 1 > 0 ? file.line - 1 : linesFrom;
            const linePrev = file.line - 2 > 0 ? file.line - 2 : linesFrom;
            const lineNext = file.line > 0 ? file.line : linesFrom;
            const lineString = editor.document.lineAt(lineCur).text;
            const lineStringPrev = editor.document.lineAt(linePrev).text;
            const lineStringNext = editor.document.lineAt(lineNext).text;
            const lineHash = (0, hash_1.getHashSumFromString)(lineString);
            const lineHashPrev = (0, hash_1.getHashSumFromString)(lineStringPrev);
            const lineHashNext = (0, hash_1.getHashSumFromString)(lineStringNext);
            if (lineHash === lineHashReportCurrent &&
                lineHashPrev === lineHashReportPrev &&
                lineHashNext === lineHashReportNext) {
                return file.line;
            }
            else {
                let lineIsFound = false;
                // Ищем нужную строку +-10 строк от изначальной
                for (let i = linesFrom; i <= linesTo; i++) {
                    const string = editor.document.lineAt(i).text;
                    const prevString = editor.document.lineAt(i - 1).text;
                    const nextString = editor.document.lineAt(i + 1).text;
                    let _lineHash = 0;
                    let _lineHashPrev = 0;
                    let _lineHashNext = 0;
                    if (string === undefined) {
                        continue;
                    }
                    _lineHash = (0, hash_1.getHashSumFromString)(string);
                    _lineHashPrev = (0, hash_1.getHashSumFromString)(prevString);
                    _lineHashNext = (0, hash_1.getHashSumFromString)(nextString);
                    if (_lineHash === lineHashReportCurrent &&
                        _lineHashPrev === lineHashReportPrev &&
                        _lineHashNext === lineHashReportNext) {
                        lineIsFound = true;
                        return i + 1;
                    }
                }
                if (!lineIsFound) {
                    return file.line;
                }
            }
        });
    }
    _getHtmlForWebview(webview) {
        // The CSS file from the React build output
        const stylesUri = (0, getUri_1.getUri)(webview, this.extensionUri, [
            "webview-ui",
            "build",
            "static",
            "css",
            "main.css",
        ]);
        // The JS file from the React build output
        const scriptUri = (0, getUri_1.getUri)(webview, this.extensionUri, [
            "webview-ui",
            "build",
            "static",
            "js",
            "main.js",
        ]);
        const codiconsUri = (0, getUri_1.getUri)(webview, this.extensionUri, [
            "webview-ui",
            "build",
            "fonts",
            "codicon",
            "codicon.css",
        ]);
        // Use a nonce to only allow a specific script to be run.
        return /*html*/ `
      <!DOCTYPE html>
      <html lang="en">
        <head>
          <meta charset="utf-8">
          <meta name="viewport" content="width=device-width,initial-scale=1,shrink-to-fit=no">
          <meta name="theme-color" content="#000000">
          <link rel="stylesheet" type="text/css" href="${stylesUri}">
          <link rel="stylesheet" type="text/css" href="${codiconsUri}">
          <title>Hello World</title>
        </head>
        <body>
          <noscript>You need to enable JavaScript to run this app.</noscript>
          <div id="root"></div>
          <script src="${scriptUri}"></script>
        </body>
      </html>
    `;
    }
}
exports.ReportPanel = ReportPanel;
ReportPanel.viewType = "PVSPanel";
//# sourceMappingURL=ReportPanel.js.map