"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSender = void 0;
const vscode = require("vscode");
class MessageSender {
    static enable() {
        this._disableValue = Math.max(this._disableValue - 1, 0);
    }
    static disable() {
        this._disableValue += 1;
    }
    static send(message, type, isModal = false, ...actionNames) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._disableValue > 0) {
                return null;
            }
            if (this._messagesBuffer.some(m => m.message === message && m.type === type)) {
                return null;
            }
            if (this._bufferClearingTimeout) {
                clearTimeout(this._bufferClearingTimeout);
            }
            this._bufferClearingTimeout = setTimeout(() => this._messagesBuffer = [], this._bufferСlearingTimeoutValue);
            this._messagesBuffer.push({ message, type });
            const func = type === "error" ? vscode.window.showErrorMessage
                : type === "warning" ? vscode.window.showWarningMessage
                    : vscode.window.showInformationMessage;
            return func(message, { modal: isModal }, ...actionNames);
        });
    }
    static getBuffer() {
        return [...this._messagesBuffer];
    }
}
exports.MessageSender = MessageSender;
MessageSender._disableValue = 0;
MessageSender._bufferСlearingTimeoutValue = 1;
MessageSender._messagesBuffer = [];
//# sourceMappingURL=MessageSender.js.map