"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyzerType = exports.AnalyzerLanguage = void 0;
const Viva64_CCpp_ErrorCode_Begin = 100;
const Viva64_CCpp_ErrorCode_End = 499;
const General_CCpp_Low_ErrorCode_Begin = 500;
const General_CCpp_Low_ErrorCode_End = 799;
const Optimization_CCpp_ErrorCode_Begin = 800;
const Optimization_CCpp_ErrorCode_End = 999;
const General_CCpp_High_ErrorCode_Begin = 1000;
const General_CCpp_High_ErrorCode_End = 1999;
const CustomerSpecific_CCpp_ErrorCode_Begin = 2000;
const CustomerSpecific_CCpp_ErrorCode_End = 2499;
const MISRA_CCpp_ErrorCode_Begin = 2500;
const MISRA_CCpp_ErrorCode_End = 2999;
const General_Cs_ErrorCode_Begin = 3000;
const General_Cs_ErrorCode_End = 3499;
const AUTOSAR_CCpp_ErrorCode_Begin = 3500;
const AUTOSAR_CCpp_ErrorCode_End = 3999;
const Optimization_Cs_ErrorCode_Begin = 4000;
const Optimization_Cs_ErrorCode_End = 4199;
const OWASP_CCpp_ErrorCode_Begin = 5000;
const OWASP_CCpp_ErrorCode_End = 5299;
const OWASP_Java_ErrorCode_Begin = 5300;
const OWASP_Java_ErrorCode_End = 5599;
const OWASP_Cs_ErrorCode_Begin = 5600;
const OWASP_Cs_ErrorCode_End = 5999;
const General_Java_ErrorCode_Begin = 6000;
const General_Java_ErrorCode_End = 6499;
exports.AnalyzerLanguage = {
    Undefined: "Undefined",
    CCpp: "cpp",
    Cs: "cs",
    Java: "java",
};
exports.AnalyzerType = {
    General: "General",
    Viva64: "64-bit",
    Optimization: "Optimization",
    CustomerSpecific: "Customer Specific",
    MISRA: "MISRA",
    AUTOSAR: "AUTOSAR",
    OWASP: "OWASP",
    Unknown: "Unknown",
};
function getAnalyzerTypeAndLang(vivaCode) {
    // Числовая часть кода
    const errorCode = parseInt(vivaCode.slice(1, vivaCode.length));
    let language;
    let type;
    if (!Number.isInteger(errorCode)) {
        language = exports.AnalyzerLanguage.Undefined;
        type = exports.AnalyzerType.Unknown;
    }
    else if (errorCode >= Viva64_CCpp_ErrorCode_Begin && errorCode <= Viva64_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.Viva64;
    }
    else if (errorCode >= General_CCpp_Low_ErrorCode_Begin &&
        errorCode <= General_CCpp_Low_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.General;
    }
    else if (errorCode >= Optimization_CCpp_ErrorCode_Begin &&
        errorCode <= Optimization_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.Optimization;
    }
    else if (errorCode >= General_CCpp_High_ErrorCode_Begin &&
        errorCode <= General_CCpp_High_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.General;
    }
    else if (errorCode >= CustomerSpecific_CCpp_ErrorCode_Begin &&
        errorCode <= CustomerSpecific_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.CustomerSpecific;
    }
    else if (errorCode >= MISRA_CCpp_ErrorCode_Begin && errorCode <= MISRA_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.MISRA;
    }
    else if (errorCode >= General_Cs_ErrorCode_Begin && errorCode <= General_Cs_ErrorCode_End) {
        language = exports.AnalyzerLanguage.Cs;
        type = exports.AnalyzerType.General;
    }
    else if (errorCode >= AUTOSAR_CCpp_ErrorCode_Begin && errorCode <= AUTOSAR_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.AUTOSAR;
    }
    else if (errorCode >= Optimization_Cs_ErrorCode_Begin &&
        errorCode <= Optimization_Cs_ErrorCode_End) {
        language = exports.AnalyzerLanguage.CCpp;
        type = exports.AnalyzerType.AUTOSAR;
    }
    else if (errorCode >= OWASP_CCpp_ErrorCode_Begin && errorCode <= OWASP_CCpp_ErrorCode_End) {
        language = exports.AnalyzerLanguage.Cs;
        type = exports.AnalyzerType.Optimization;
    }
    else if (errorCode >= OWASP_Java_ErrorCode_Begin && errorCode <= OWASP_Java_ErrorCode_End) {
        language = exports.AnalyzerLanguage.Java;
        type = exports.AnalyzerType.OWASP;
    }
    else if (errorCode >= OWASP_Cs_ErrorCode_Begin && errorCode <= OWASP_Cs_ErrorCode_End) {
        language = exports.AnalyzerLanguage.Cs;
        type = exports.AnalyzerType.OWASP;
    }
    else if (errorCode >= General_Java_ErrorCode_Begin && errorCode <= General_Java_ErrorCode_End) {
        language = exports.AnalyzerLanguage.Java;
        type = exports.AnalyzerType.General;
    }
    else if (errorCode >= 1 && errorCode <= 50) {
        language = exports.AnalyzerLanguage.CCpp;
        type = errorCode === 4 ? exports.AnalyzerType.Viva64 : exports.AnalyzerType.Unknown;
    }
    else if (errorCode === 51 || errorCode === 52) {
        language = exports.AnalyzerLanguage.Cs;
        type = exports.AnalyzerType.Unknown;
    }
    else if (errorCode >= 0 && errorCode < 100) {
        language = exports.AnalyzerLanguage.Undefined;
        type = exports.AnalyzerType.Unknown;
    }
    else {
        language = exports.AnalyzerLanguage.Undefined;
        type = exports.AnalyzerType.Unknown;
    }
    return { type, lang: language };
}
exports.default = getAnalyzerTypeAndLang;
//# sourceMappingURL=getAnalyzerTypeAndLang.js.map