"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAnalyzerErrorCode = exports.getHashSumFromString = exports.getStringWithoutDiagnosticComment = void 0;
/**
 * Убирает из строки комментарий вида "//-V505"
 * @param str Строка файла
 * @returns
 */
function getStringWithoutDiagnosticComment(str) {
    let trimmedString = str.trim();
    const indexOfComment = trimmedString.lastIndexOf("//-V");
    if (indexOfComment !== -1) {
        trimmedString = str.slice(0, indexOfComment);
    }
    return trimmedString.trim();
}
exports.getStringWithoutDiagnosticComment = getStringWithoutDiagnosticComment;
/**
 * Возвращает хэш сумму строки по алгоритму, взятому из PVS-Studio_Cmd
 * @param str
 * @returns
 */
function getHashSumFromString(str) {
    let continueSearch = true;
    do {
        continueSearch = false;
        let index = str.lastIndexOf("//-");
        if (index !== -1) {
            let subStr = str.substring(index + 3);
            subStr = subStr.trimEnd();
            continueSearch = isAnalyzerErrorCode(subStr);
            if (continueSearch) {
                str = str.replaceAll(subStr, "");
            }
            else {
                str = str.replaceAll("//-", "");
            }
        }
    } while (continueSearch);
    let sum = 0;
    for (let char of str) {
        const charCode = char.charCodeAt(0);
        if (charCode >= 0 && char !== " " && char !== "\t") {
            const hiBit = (sum & 0x80000000) !== 0;
            sum <<= 1;
            sum = sum & 0xffffffff; // костыль, чтоб не переполнить 32 бита
            sum ^= charCode;
            if (hiBit) {
                sum ^= 0x00000001;
            }
        }
    }
    return sum;
}
exports.getHashSumFromString = getHashSumFromString;
function isAnalyzerErrorCode(errStr) {
    if (errStr.length !== 4 && errStr.length !== 5) {
        return false;
    }
    if (errStr[0] !== 'V' && errStr[0] !== 'v') {
        return false;
    }
    for (var i = 1; i <= 3; i++) {
        if (isNaN(Number(errStr[i]))) {
            return false;
        }
    }
    if (errStr.length === 5) {
        if (isNaN(Number(errStr[4]))) {
            return false;
        }
    }
    return true;
}
exports.isAnalyzerErrorCode = isAnalyzerErrorCode;
//# sourceMappingURL=hash.js.map