"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.saveFile = void 0;
const fs = require("fs");
const path = require("path");
const vscode = require("vscode");
const _fileSavingErrorMessage = "";
function saveFile(data, fsPath, showMessageWhenError = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const pathWithoutFile = path.dirname(fsPath);
        if (!fs.existsSync(pathWithoutFile)) {
            yield fs.promises.mkdir(pathWithoutFile, { recursive: true });
        }
        const stringifiedData = JSON.stringify(data, undefined, 2);
        const writeStream = fs.createWriteStream(fsPath);
        const response = new Promise((resolve) => {
            writeStream.write(stringifiedData, (error) => {
                if (error) {
                    if (showMessageWhenError) {
                        vscode.window.showErrorMessage(`Failed to create or overwrite file: \"${fsPath}\".`);
                    }
                    resolve(error);
                }
                else {
                    resolve(undefined);
                }
                writeStream.end();
            });
        });
        return response;
    });
}
exports.saveFile = saveFile;
//# sourceMappingURL=saveFile.js.map